/** 
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: k5vsi-vdif-dim_vdifpro.cpp 56 2013-07-03 11:40:14Z vlbi $ 
 */


#ifndef LINUX
#define LINUX
#endif
#ifndef _LARGEFILE_SOURCE 
#define _LARGEFILE_SOURCE
#endif 
#define _LARGEFILE64_SOURCE

#include <cstdio>
#include <cstdlib>
#include <string>
using namespace std;

extern "C"{
  //#include <getopt.h>
}

#include "time_lib.hpp"

#include "k5vsi_mdl.hpp"
#include "VDIF_Proto.hpp"
#include "VLBI_Frame.hpp"
#include "Dumper.hpp"

#define DEBUG(x) 

char *me;  // My name

// int Str2Tint(char *a){
//   int scale=1;
  
//   if (a[strlen(a)-1]=='h') {
//     scale=60*60; /* Hours -> seconds  */
//   } else if (a[strlen(a)-1]=='m') {
//     scale=60; /* Minutes -> seconds  */
//   }
//   int length=(uint32_t) ceil(atof(a)*scale);
//   return length;
// }

void usage(){
  printf("Usage: %s [options]\n",me);
  printf("  -s <Byte>        :  UDP block size\n");
  //  printf("  -f <vdif/m5b/k5> :  Data Frame type. Default(vdif)\n");
  printf("  -t <xs|xm|xh>    :  time length to sent\n");
  printf("  -p <PORT>        :  Port to use\n");
  printf("  -q <udp/tcp>     :  Protocol used for net transfer\n");
  printf("  -a <HOSTNAME>    :  Remote host name\n");
  printf("  -m <'xxMsps/xbit/xxch'>: Data mode\n");
  printf("        Msps -> [04Msps|08Msps|16Msps|32Msps|64Msps]\n");
  printf("        bit  -> 2bit only                       \n");
  printf("        ch   -> [16ch|08ch|04ch]                \n");
  printf("  -o <out file name>: Writing data to a file. \n");
  printf("  -g <igrp>        : Group number 0-32/nch, default(0).\n");
  //  printf("  -v <np>          :  Verbose level (default=0).\n");
  printf("  -h                    This list\n");
  exit(1);
}

int main (int argc, char * const argv[]) {
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  
  /// Device Info Setup 
  K5_VSI k5vsi;  // K5VSI device module 
  k5vsi.Set_Mode("16Msps/2bit/08ch");

  //  VDIF_Frame vdifr; // Using VDIF frame 

  //  string outfname;
  string hostname;
  string proto="udp"; //< Protocl (udp/tcp) for data transfer over the net. 
  int port=DEFAULT_PORT;
  int mtu  = DEFAULT_MTU;
  if(argc < 2) usage();
  int tspan = 11;
  
  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hs:t:p:a:m:o:g:q:")) { 
    case 'h':   usage();     break;
    case 'p':   port=atoi(optarg); break;
    case 'q':   proto = optarg; break;
    case 'a':   hostname = optarg;      break;
    case 't':   tspan = Str2Tint(optarg); break;
    case 'g': { 
      int work=atoi(optarg); 
      k5vsi.Set_iGrp(work); break;}
    case 'm':   
      if( k5vsi.Set_Mode(optarg) >0) 
	fprintf(stderr,"Mode=%s\n",optarg); 
      break;
    case 'o':  hostname =optarg; proto="file";  break;
      //    case 'f':   outfname =optarg; break;
    case 's':  mtu = atoi(optarg); break;
      //    case 'v':   Logsystem::debug_logger()->Set_level(atoi(optarg)) ;    break;
    case '?':  usage(); break;
    }
  }
  
  if(hostname.empty()){
    cerr << "No Output (file/net) Specified\n";
    usage();
  }
  
  /// Setup protocol Output interface   
  // VLBI_Proto *tx_proto;
  VDIF_Proto vdif_pro(port); // Using VDIF Header and protocol 
  //  FLIO_Proto flio_pro();
  
  if(proto.find("udp",0,3) != proto.npos){
    vdif_pro.Create_sock(VLBI_Proto::UDP_TX,mtu); 
    printf(" Data (%s) Output to %s by Net(UDP) \n", 
	   k5vsi.Mode().c_str(),hostname.c_str());
  };
  if(proto.find("tcp",0,3) != proto.npos){
    vdif_pro.Create_sock(VLBI_Proto::TCP_TX,mtu); 
    printf(" Data (%s) Output to %s by Net(TCP) \n", 
	   k5vsi.Mode().c_str(),hostname.c_str());
  };
  if(proto.find("file",0,4) != proto.npos){
    vdif_pro.Create_sock(VLBI_Proto::FILE_IO,mtu); 
    printf(" Data (%s) Output to FILE %s \n", 
	   k5vsi.Mode().c_str(),hostname.c_str());
  };

  vdif_pro.Set_Host(hostname.c_str(),port);

  /// Setup data format/frame 
  VDIF_Frame vdif;
  vdif.Set_Sparam(k5vsi.Nch(), k5vsi.Nbit(),k5vsi.Srate());
  vdif.Set_StnID("kb");
  vdif.Set_FrameDiv(10);
  
  if(  vdif_pro.Ready(&vdif) < 0){
    cerr << "Error: VDIF-Protocol is not ready\n"; exit(1); };
			  
  Dumper dumper;
  // Attaching IO interfaces
  dumper.Set_Span(tspan);

  if(dumper.Set_IO(&vdif_pro, &k5vsi) <0){
    cerr << "Error Dumper::Set_IO()\n";
    return -1;
  }

  dumper.Ready();
  dumper.start();

  //  while(1){     ; }
  if(dumper.join() !=0){
    cerr << "dumper returned with error.\n";
  }else
    cerr << "dumper returned with success.\n";

  return(0);
}

