#!/usr/bin/perl -w 

sub usage(){
    print STDERR "Usage: $0 -a <host>[:<port>] -odir <dir>   \n";
    print STDERR " \t [ -mode <Msps>:<bit>:<Nch>] [-id <ID>] [-fn <nun>]. \n";
    print STDERR " Rx start for sudp_recv3. \n";
    print STDERR "  -id        : Station ID, default = 'O'.\n";
    print STDERR "  -mode         : Recording mode, default = '16:1:16'.\n";
    print STDERR "  -odir <dir>: Output dir.\n";
    print STDERR "  Rx server port: default=9903. \n";
    print STDERR " -fn: File naming convention: (default 2)\n";
    print STDERR "     1 : XDDDNNNN.dat (default)\n";
    print STDERR "    -1 : XDDDNNNN.#ch.dat \n";
    print STDERR "     2 : sidDDDHHMMSSG.dat \n";
    print STDERR "    -2 : sidDDDHHMMSSG.#ch.dat\n";
    exit(1);
}

# use Exporter;
use IO::Socket;                   # IO::Socket ⥸塼Ȥ
use lib "$ENV{HOME}/prog/perl/module";
use Time_lib;

my @cmds= qw/"in2net=connect:192.42.120.66" "mode=ext:0xff:1" "play_rate=data:08" "net_protocol=UDP:3200000:10" "in2net=on"/;

my $cnt=0;
{
    usage() if(@ARGV <5);
    my $port = 9903;
    my $host = "localhost";
    my $stid = "O";
    my $msps = 16;
    my $bit = 1;
    my $nch = 16;
    my $odir;
    my $fn=2;
    while(@ARGV){
	my $opt = shift;
	if($opt =~/-a/){
	    my @terms = split(/:/,shift);
	    $host= $terms[0];
	    $port = $terms[1]  if(@terms > 1);
	}elsif($opt=~/-mode/){
	    my @terms = split(/:/,shift);
	    if(@terms < 3){
		print "E -m setting is wrong\n";
		usage();
	    }
	    $msps=$terms[0];
	    $bit =$terms[1];
	    $nch =$terms[2];
	}elsif($opt=~/-id/){
	    $stid = shift;
	}elsif($opt=~/-odir/){
	    $odir  = shift;
	}elsif($opt=~/-fn/){
	    $fn = shift;
	}
    }

    my $ngrp = 1;
    my $mch=1;
    if($nch == 1 ){
	$ngrp = 1;
	$mch = 1;
    }elsif( ($nch % 4) == 0 ){
	$ngrp = $nch /4 ;
	$mch = 4;
    }else{
	print "E Nch must be (1,4,8,16)\n";
	usage();
    }

    my $remote = IO::Socket::INET->new(Proto    => "tcp",
				       PeerAddr => $host,
				       PeerPort => $port );
    
    if( ! $remote){	die "Could not connect to $host:$port \n";    }

    select($remote); $|=1; # stop buffering 
    select(STDIN); $|=1; # stop buffering 
    select(STDOUT); $|=1; # stop buffering 

    &communicate($remote,"data_out=multi:$ngrp;");
    for(my $i =0;  $i < $ngrp; $i++){
	my $mode = sprintf "%d:%d:%d:%d:%d", $i, $msps, $bit, $mch, $i*$mch;
#	print "cmd:  modegrp=$mode;";
	system("usleep 100000");
	&communicate($remote,"modegrp=$mode;");
    }
   my @fname;
    for(my $igrp=0; $igrp<$ngrp; $igrp++){
	$fname[$igrp] = $odir . "/"
	    . &Naming($stid,$igrp,$fn);
	my $outspec = sprintf("data_out=FILE%d:%s",
			      $igrp,$fname[$igrp]);
#	print "$outspec \n";
#	usleep(100000);
	system("usleep 100000");
	&communicate($remote,$outspec);
    }


    system("usleep 100000");
    ## Rx start 
    $msg = "in2net=on;";
    print "Cmd: $msg\n";
    &communicate($remote,"$msg\n");

    close $remote;
}

sub communicate($$){
    my $socket = shift;
    my $line = shift;

    print "SND:$line\n";
    last unless(defined($socket));
    print $socket "$line\n";
    last unless(defined($socket));
    
    my $res = <$socket>;
    next unless(defined($res));
    chomp($res);
    print "Res[$cnt]: $res\n" if(defined($res));
    $cnt++;
    
}

sub Naming($$$;$){
    my ( $id, $igrp,$fn) = @_;
    $fn = 2 unless(defined($fn));
    my $t1 = new Time_lib;
    $t1->Set_time_now();
    my $doy = $t1->Doy();
    my @grp = qw(a b c d);
    my @ydhms = $t1->YDHms();
    my $fname;    
    if($fn == 1){
	$fname = sprintf("%s%03d%04d.dat",$id,$doy,$i);
    }elsif($fn == -1){
	$fname = sprintf("%s%03d%04d.%d.dat",$id,$doy,$igrp,$i);
    }elsif($fn == -2){
	$fname = sprintf("%s%03d%02d%02d%02d%s.%d.dat",
			 $id,$ydhms[1],$ydhms[2],$ydhms[3],$ydhms[4],
			 $grp[$igrp], $igrp);
    }else{
	$fname = sprintf("%s%03d%02d%02d%02d%s.dat",
			 $id,$ydhms[1],$ydhms[2],$ydhms[3],$ydhms[4],
			 $grp[$igrp]);
    }
    return $fname;
}


