/** 
 * @Author M.Sekido
 * @Version $Rev: 56 $
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: test-sudp_send.cpp 56 2013-07-03 11:40:14Z vlbi $ 
 */
#include <iostream>
#include <cstdio>
#include <vector>
#include <string>
#include <cstring>
#include <cstdlib>
using namespace std;

#include "SUDPProto.hpp"
#include "tim_util.h"

// extern "C" {
// #include <unistd.h>
// #include <signal.h>
// #include <sys/time.h>
// }


char *me;  // My name

// void my_usleep(u_int64_t usec) ;
// int aflag=0;
// void alarm_signal (int sig) {
//   aflag=1;
//   return;
// }  

void usage(){
  fprintf(stderr, "Usage: %s [-f <filename> | -m] [-a <host>] [-p <port>][-s <size>] \n", me);
  fprintf(stderr, "   [-r <Mbps>]\n");
  fprintf(stderr,"  -f: sending file with SUDP packets.\n");
  fprintf(stderr,"  -m: sending dummy data with UDP packets.\n");
  fprintf(stderr,"  -s: Packet size(mtu). default=1004 Byte. Payload = mtu-4.\n");
  fprintf(stderr,"  -p: Port number (default:9901)\n");
  fprintf(stderr,"  -r: Rate Mbps (default:100)\n");
  fprintf(stderr,"  -a: hostname (default:localhost)\n");
  exit(1);
}

int main(int argc, char *argv[]){
  me = (me = strrchr(argv[0], '/')) == NULL ? argv[0] : me+1; /* My name */ 
  string fname;
  string hostname("localhost");
  int port(9901);
  int mflag(0);
  int mtu(1004);  
  int rate(100); // Mbps 

  int k=0;
  while (k >= 0) { 
    switch (k = getopt(argc, argv, "hf:a:p:ms:r:")) { 
    case 'h':   usage();     break;
    case 'f':   fname = optarg; break;
    case 's':   mtu = atoi(optarg); break;
    case 'r':   rate = atoi(optarg); break;
    case 'm':   mflag=1; break;
    case 'a':   hostname = optarg; break;
    case 'p':   port  = atoi(optarg); break;
    case '?':    usage(); break;
    }
  }
  if(mflag==0 && fname.empty()){
    cerr << "Error: Filename is not specified\n";
    usage();
  }
  
  SUDPProto  sudp(mtu);
  if(sudp.Set_Dest_Host(hostname.c_str(), port) < 0){
    perror("Error in SUDPProto.Set_Host()\n");
    usage();
  }
  
  sudp.Set_Mbps(rate); // 100 Mbps

  FILE *fp;

  sudp.Time_Start(); //  Timer Start 

  if(fname.empty()){ // sending test data from memory
    int32_t psize = rate*1000000/8;
    vector <uint8_t> buf(psize);
    cerr << " Transmission from Memory starterd\n";
    for(int i=0; i<100; i++){
      //      int32_t sent = 
      sudp.TX(&buf[0],psize);
      //      fprintf(stdout," Sent data=%ld\n", sent);
    }
  }else{   // sending from file.
    if((fp= fopen(fname.c_str(), "r")) ==NULL){
      cerr << "Error: Filed to open file(" << fname << ") \n";
      exit(1);
    }
    fprintf(stderr," File(%s) transfer starterd\n", fname.c_str());
    int32_t sent = sudp.TX(fp);
    fprintf(stdout," Sent data=%ld\n", (long int)sent);
    fclose(fp);  
  }
}


