#ifndef __K5VSSP_HEADER__
#define __K5VSSP_HEADER__

// #include "VLBI_Header.hpp"
/** K5VSSP32 Class Library Header.
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: K5VSSP_Header.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include <cstdio>
#include <cstdlib>
#include <iostream>
using namespace std;

#include "Logsystem.hpp"
#include "time_lib.hpp"

extern "C"{
#include <stdint.h>
}

const int K5VSSP_Hdr_size = 8;
const int K5VSSP32_Hdr_size = 32;

//========== VSSP32 ===From here ====================

/// Structure of K5/VSSP aux field format #1
struct K5_Aux_1{
  uint8_t aux_fmt_No; ///< Aux field format number
  uint8_t fltr_freq; ///< Filter Freq(MHz)
  //-----------------
  char station_id[2]; ///< Station ID
  //-----------------
  char stname[8]; ///< Host PC name
  char hostPCname[8]; ///< Host PC name
public:
  /// Set Station ID
  void Set_StnID(const char* id){aux_fmt_No=1; strncpy(station_id,id,2);}
  /// Set Station name
  void Set_Stname(const char* nm){aux_fmt_No=1; strncpy(stname,nm,8);}
  /// Set Hostname
  void Set_Host(const char* hs){aux_fmt_No=1; strncpy(hostPCname,hs,8);}
};

/// Structure of K5/VSSP aux field format #2
struct K5_Aux_2{
  uint8_t aux_fmt_No; ///< Aux field format number
  uint8_t fltr_freq; ///< Filter Freq(MHz)
  //-----------------
  char dummy[10]; ///< Dummy data, 0x5555 
  //-----------------
  char hostPCname[8]; ///< Host PC name
};

/// Structure of K5/VSSP aux field format #85 and #170
struct K5_Aux_85{
  uint8_t aux_fmt_No; ///< Aux field format number
  uint8_t fltr_freq; ///< Filter Freq(MHz)
  //-----------------
  char dummy[18]; ///< Dummy data, 0x5555 for #85, 0xAAAA for #170
};

/// Structure of K5VSSP header 
//class K5VSSP_Hdr:public VLBI_Header {
class K5VSSP_Hdr {
  uint32_t sync_word; ///< sync word ffff
  unsigned int sod:17; ///< sod 
  //  unsigned sec1:1;  ///< MSB 1bit of sod
  unsigned nch_b:1; ///< nch data
  unsigned sfreq_b:4; ///< Sampling freq data
  unsigned qbit_b:2;  ///< Qunatization bit data
  //  uint8_t  sync2;   ///< 2nd Sync word 0x8b or 0x8c
  uint8_t   sync2;   ///< 2nd Sync word 0x8b or 0x8c
public:
  double Sod()const{return (double)sod;} ///< returns Sec of the day;
  int Nch() const {if(nch_b) return 4; else return 1;}   ///< Returns Nch
  int Qbit()const;  ///< returns Qunatization bits
  uint32_t Sfreq() const; ///< Returns Sampling freq.
  /// Testing of header matching, returns 1 :VSSP32, 2: VSSP
  int HD_Match() const{  return sync_word != 0xffffffff ? 0 : 
      (sync2 == 0x8b ? 1 : 2) ;}
  int Headsize() const{return K5VSSP_Hdr_size;} ///< Return Headsize 
  int Framesize();  ///< Header Framesize (bytes) including header
  uint32_t Epoch_imjd();    ///< return mjd in integer
  double Epoch_sod()  const{return (double)sod;} ///< returns Sec of the day;
  void Print_hd(FILE *fp=stdout); ///< Print Header info
  const int Set_Nch(const int n);   ///< Set Number of channels
  const int Set_Qbit(const int q);    ///< Set Quantization bit
  const int Set_Srate(const int32_t s);   ///< Set Sampling rate (Hz)
  /// Set Epoch
  void Set_Epoch(const Time_dat & t1){sod = (int)t1.Sod();}
};



inline uint32_t  K5VSSP_Hdr::Epoch_imjd() {
  cerr << "Warning: K5VSSP_Hdr::Epoch_imjd() is not implemented.";
  cerr << " Returning 0day\n";
  return 0;
}

// inline VLBI_Hdr_t K5VSSP_Hdr::Header_type()const {
//   switch( sync2){
//   case  0x8b:  return K5VSSP_Header_Type; break;
//   case  0x8c:  return K5VSSP32_Header_Type; break;
//   default:     return UN_KNOWN; break;
//   }
// }

// inline int  K5VSSP_Hdr::Headsize() const{
//   switch (sync2){
//   case  0x8b: return 8; break;
//   case  0x8c: return 32; break;
//   default: 
//     cerr << "Error in K5VSSP_Hdr::Headsize(): sync2 is unexpected\n";
//     return 0; break;
//   }
// }

inline int  K5VSSP_Hdr::Framesize() {
  int hdsize = Headsize();
  int datasize = Sfreq()/8* Qbit()* Nch();
  return (datasize + hdsize);
}


/// Structure of K5VSSP header 
//struct K5VSSP32_Hdr:public VLBI_Header {
struct K5VSSP32_Hdr {
  uint32_t sync_word; ///< sync word ffff
  unsigned int sod:17; ///< sod 
  //  unsigned sec1:1;  ///< MSB 1bit of sod
  unsigned nch_b:1; ///< nch data
  unsigned sfreq_b:4; ///< Sampling freq data
  unsigned qbit_b:2;  ///< Qunatization bit data
  //  uint8_t  sync2;   ///< 2nd Sync word 0x8b or 0x8c
  uint8_t   sync2;   ///< 2nd Sync word 0x8b or 0x8c
  unsigned doy:9;   ///< Day of Year
  unsigned year:6;  ///< year 
  unsigned eflag:1; ///< flag 
  uint8_t  aux_size; ///< aux size (Byte)
  unsigned ver_mn:4; ///< Major Version 
  unsigned ver_mj:4; ///< Minor Version
  char   ext_aux[20]; ///< Aux data
  //  K5_Aux_1 aux_data;///< Aux data
public:
  /// Constructor
  K5VSSP32_Hdr():sync_word(0xffffffff),sync2(0x8c),aux_size(20),ver_mj(1){
    memset(ext_aux,0,20);}
  void Disp(); ///< Display header info to stderr
  double Sod()const{return (double)sod;} ///< returns Sec of the day;
  int Nch() const {if(nch_b) return 4; else return 1;}   ///< Returns Nch
  int Qbit()const;  ///< returns Qunatization bits
  uint32_t Sfreq() const; ///< Returns Sampling freq.
  int Doy()const {return (int)doy;} ///< Return total Day of year
  int Year() const {return (int) (year+2000);} ///< Returns Year
  /// Returns Filetr frequency (MHz)
  int Fltr_freq()const{K5_Aux_1 *p=(K5_Aux_1*)ext_aux; return p->fltr_freq;}
  void Print_Aux_Info(FILE *fp=stderr); ///< Print Aux info
  //  VLBI_Hdr_t Header_type()const;
  /// Testing of header matching, returns 1 :VSSP32, 2: VSSP
  int HD_Match() const{  return sync_word != 0xffffffff ? 0 : 
      (sync2 == 0x8b ? 1 : 2) ;}
  int Framesize(); ///< returns Framesize 
  /// Returns Header size (Byte)
  int Headsize() const{return K5VSSP32_Hdr_size;}
  Time_dat Epoch(); ///< return Time_dat data of the epoch
  /// Return integer MJD of the epoch
  uint32_t Epoch_imjd(){return (uint32_t)(floor(Epoch().Mjd()));}
  /// Returns Second of the day
  double Epoch_sod()  const{return (double)sod;}
  void Print_hd(FILE *fp=stdout); ///< Print Header info
  /// Set Nch=1,4, otherwise, return -1;
  const int Set_Nch(const int n);  ///< Set Number of channels
  const int Set_Qbit(const int q);   ///< Set Quantization bit 
  const int Set_Srate(const int32_t s); ///< Set Sampling Rate (Hz)
  void Set_Epoch(const Time_dat & t1);   ///< Set Epoch
  /// Set Station ID
  void Set_StnID(const char* id){
    K5_Aux_1* paux=(K5_Aux_1*)ext_aux; paux->Set_StnID(id);}
  /// Set Station name
  void Set_Stname(const char* nm){
    K5_Aux_1* paux=(K5_Aux_1*)ext_aux; paux->Set_Stname(nm);}
  /// Set Hostname
  void Set_Host(const char* hs=0);
  /// Get Station Name
  void Get_Stname(string& name);
  /// Get Station ID
  void Get_StnID(string& id);
};

inline void K5VSSP32_Hdr::Get_Stname(string& name){
  K5_Aux_1* paux=(K5_Aux_1*)ext_aux;
  if(paux->aux_fmt_No==1){
    char buf[10];
    strncpy(buf,paux->stname,8); buf[8]='\0';
    name = buf;
  }
}

inline void K5VSSP32_Hdr::Get_StnID(string& id){
  K5_Aux_1* paux=(K5_Aux_1*)ext_aux;
  if(paux->aux_fmt_No==1){
    char buf[3];
    strncpy(buf,paux->station_id,2); buf[2]='\0';
    id = buf;
  }
}

inline  void K5VSSP32_Hdr::Set_Host(const char* hs){
  K5_Aux_1* paux=(K5_Aux_1*)ext_aux; 
  if(hs==0){
    char buf[80];    gethostname(buf,80);
    paux->Set_Host(buf);
  }else
    paux->Set_Host(hs);
}

inline void K5VSSP32_Hdr::Set_Epoch(const Time_dat & t1){
  sod = (unsigned int)t1.Sod();
  year = t1.Year() - 2000;
  doy = (unsigned int)t1.Doy();

  if(Logsystem::Test_Logsys(Logsystem::DEBUG))  {
    string func_id("K5VSSP32_Hdr::Set_Epoch():");
    string tmstr = t1.Print_time(20);
    Logsystem::debug_logger()->Tstamp() 
      << "D " << func_id << "t1=(" << tmstr << "), year="
      << year << "\n";
    Logsystem::debug_logger()->Flush();
  }

}

K5VSSP32_Hdr*   K5VSSP32_Hdr_find(const uint8_t*  buf, 
				  const uint32_t nbyte, int& offset);
K5VSSP32_Hdr*   K5VSSP32_Hdr_find(FILE *fp, int& offset);

// /// Seconds of the day
// inline double  K5VSSP_Hdr::Sod() const{
//   double sod(0);
//   if( sec1) sod = (double)(1<<16);
//   sod += sec;
//   return sod;
// }

// inline VLBI_Hdr_t K5VSSP32_Hdr::Header_type()const {
//   switch( sync2){
//   case  0x8b:  return K5VSSP_Header_Type; break;
//   case  0x8c:  return K5VSSP32_Header_Type; break;
//   default:     return UN_KNOWN; break;
//   }
// }

/// Quantization bits
inline int K5VSSP_Hdr::Qbit() const {
  int qbit=1;

  switch (qbit_b){
  case 0:  qbit = 1;    break;
  case 1:  qbit = 2;    break;
  case 2:  qbit = 4;    break;
  case 3:  qbit = 8;    break;
  default: qbit=0;    break;
  }
  return(qbit);
}

/// Quantization bits
inline int K5VSSP32_Hdr::Qbit() const {
  int qbit=1;

  switch (qbit_b){
  case 0:  qbit = 1;    break;
  case 1:  qbit = 2;    break;
  case 2:  qbit = 4;    break;
  case 3:  qbit = 8;    break;
  default: qbit=0;    break;
  }
  return(qbit);
}

/// Sampling Frequency  (Hz)
inline uint32_t  K5VSSP_Hdr::Sfreq()const {
  uint32_t Fsampling=0;

  switch( sfreq_b ){
  case 0:   Fsampling = 40000;     break;
  case 1:   Fsampling = 100000;    break;
  case 2:   Fsampling = 200000;    break;
  case 3:   Fsampling = 500000;    break;
  case 4:   Fsampling = 1000000;   break;
  case 5:   Fsampling = 2000000;   break;
  case 6:   Fsampling = 4000000;   break;
  case 7:   Fsampling = 8000000;   break;
  case 8:   Fsampling = 16000000;  break;
  case 9:   Fsampling = 32000000;  break;
  default:  Fsampling = 0;         break;
  }
  return (Fsampling);
}

/// Sampling Frequency  (Hz)
inline uint32_t  K5VSSP32_Hdr::Sfreq()const {
  uint32_t Fsampling=0;

  switch( sfreq_b ){
  case 0:   Fsampling = 40000;     break;
  case 1:   Fsampling = 100000;    break;
  case 2:   Fsampling = 200000;    break;
  case 3:   Fsampling = 500000;    break;
  case 4:   Fsampling = 1000000;   break;
  case 5:   Fsampling = 2000000;   break;
  case 6:   Fsampling = 4000000;   break;
  case 7:   Fsampling = 8000000;   break;
  case 8:   Fsampling = 16000000;  break;
  case 9:   Fsampling = 32000000;  break;
  case 10:  Fsampling = 64000000;  break;
  case 11:  Fsampling = 128000000;  break;
  case 12:  Fsampling = 256000000;  break;
  case 13:  Fsampling = 512000000;  break;
  case 14:  Fsampling = 1024000000;  break;
  case 15:  Fsampling = 2048000000;  break;
  default:  Fsampling = 0;         break;
  }
  return (Fsampling);
}

// inline int  K5VSSP32_Hdr::Headsize() const{
//   switch (sync2){
//   case 0x8b: return 8; break;
//   case 0x9c: return 32; break;
//   default: 
//     cerr << "Error in K5VSSP_Hdr::Headsize(): sync2 is unexpected\n";
//     return 0; break;
//   }
// }

inline int  K5VSSP32_Hdr::Framesize() {
  int hdsize = Headsize();
  int datasize = Sfreq()/8* Qbit()* Nch();
  return (datasize + hdsize);
}


// /// @class K5VSSP_Hdr is header class of VSSP32
// class K5VSSP_Hdr: public VLBI_Header{
//   K5VSSP_Hdr *pt_head;
//   int dbg_disp_level; ///< Display level for Debug
//   int year, doy; ///< Year and Doy for VSSP old format
// public:
//   /// Default Constructor 
//   K5VSSP_Hdr():VLBI_Header(K5VSSP32_Hdr_size),pt_head(0),dbg_disp_level(0), 
// 		  year(0),doy(0){Init();}
//   /// Initilizer
//   void Init(){memset(&head_buf[0],0, K5VSSP32_Hdr_size);}
//   void Set_dbg(const int level){dbg_disp_level = level;}
//   /// Header finder 
//   /// @return  Number of bytes to find the header, or -1: for Error
//   int32_t Hd_find(const uint8_t*  buf, const uint32_t nbyte);
//   /// Print Header Information
//   void Print_hd(FILE *fp=stdout);
//   /// return framesize including header (bytes)
//   int Framesize() ; 
//   int Headsize()const;  ///< return header size (bytes)
//   Time_dat Epoch(); ///< return Time_dat data of the epoch
//   /// Return integer MJD of the epoch
//   uint32_t Epoch_imjd(){return (uint32_t)(floor(Epoch().Mjd()));}
//   /// Returns Second of the day
//   double Epoch_sod() {return pt_head->Sod();}  ///< return sec of the day (sec)  
//   /// Return header type
//   VLBI_Hdr_t Header_type()const;
// };

// inline int K5VSSP_Hdr::Headsize()const {
//   if(pt_head==0) {cerr << "Error: K5VSSP header is not yet set\n";  return 0;}
//   switch( pt_head->sync2){
//   case  0x8b:  return K5VSSP_Hdr_size; break;
//   case  0x8c:  return K5VSSP32_Hdr_size; break;
//   default:     return 0; break;
//   }
//   //   int hsize = sizeof(K5VSSP_Hdr);
//   //   if( hsize != K5VSSP_Hdr_size) 
//   //     cerr << "Error size of K5VSSP_Hdr(" << hsize
//   // 	 <<  ") != 32 " << endl;
//   //   return (hsize);
// } 



//========== VSSP32 ===to Here  ====================


#endif
