/** 
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: VDIF_Frame.cpp 56 2013-07-03 11:40:14Z vlbi $ 
 */


#include "VDIF_Frame.hpp"
extern "C"{
#include <errno.h>
#include <arpa/inet.h>
}

VDIF_Frame::VDIF_Frame():VLBI_Frame(DEFAULT_MTU_SIZE),fp(0),
			 workbuf(0),nframe_sec(0),hdr_tested(-1){
    Time_dat t1(2000,1,1,0,0,0);
    ref_epoch0 = t1;
    vdif_hdr.Set_Nch(das_inf.Nch());
    vdif_hdr.Set_Qbit(das_inf.Qbit());
    vdif_hdr.Set_Srate(das_inf.Srate()/1000);
}


const long VDIF_Frame::Read_every_sec_hdr(vector <VDIF_Hdr>& hdrarr,
					  const size_t nsec,
					  const Read_t mode){

  if(fp==0){
    cerr << "E Failed VDIF_Frame::Test_Read(): File is not opened yet\n";
    return -1;
  }

  long offset(0); /// First reading Header
  if(! VDIF_Hdr_find(fp, offset, &vdif_hdr)){
    cerr << "Error: Failed to find VDIF-Header \n";
    return -1;
  }; 
  hdr_tested = 0; ///< Set test-flag to 'OK'
  hdrarr.resize(0);
  framesize = vdif_hdr.Framesize();
  char dummy[framesize];
  VDIF_Hdr* phdr = (VDIF_Hdr*)dummy;

  long sec_dat[2] = {vdif_hdr.Sec_Ref(),0}; // the first.
  hdrarr.push_back(vdif_hdr);

  // Search for the next 1sec
  int sec_count(0);
  long frm_sec_cnt=-1;
  long frm_sec0=0;
  //  long frm_intvl(0);
  long frm_count(-1);
  nframe_sec =0;
  while( nsec == 0 || hdrarr.size() < nsec ){
    if(!VDIF_Hdr_find(fp, offset, phdr)) break;
    fseek(fp,framesize,SEEK_CUR);
    if(offset != 0) hdr_tested++;
    frm_count++;
    frm_sec_cnt++;
    if( (sec_dat[1] = phdr->Sec_Ref()) != sec_dat[0]){
      hdrarr.push_back(*phdr);
//       if(sec_dat[1] - sec_dat[0] == 1)
// 	frm_intvl = hdrarr[sec_count].iFrame() - phdr->iFrame();
      sec_dat[0] = sec_dat[1];
      sec_count++;    
      
      if(frm_sec0 == 0) // First 
	frm_sec0 = frm_sec_cnt; 
      else {  // Later than fist
	if(nframe_sec != 0 || frm_sec0 == frm_sec_cnt){
	  nframe_sec = frm_sec0;
	  if(mode == QUICK)  
	    fseek(fp,framesize*(nframe_sec - 1),SEEK_CUR); // Skip 1sec
	}else
	  frm_sec0 = frm_sec_cnt; 
      } // else
      frm_sec_cnt=0;
    };  // if
  } // while
  //  if(frm_intvl >0) nframe_sec = frm_intvl;
  //  nframe_sec = frm_count/sec_count;

  // Set Srate for reference for non-NICT version
  if(hdrarr[0].exud_nict.ext_dat_ver != 1 && frm_count>0){ 
    long datsize = Framesize() - Headsize();
    int nch = hdrarr[0].Nch();
    int qbit = hdrarr[0].Qbit();
    uint32_t srate_ksps = frm_count * datsize * 8/hdrarr.size()/qbit/nch/1000;
    vdif_hdr.Set_Srate(srate_ksps);
    for(int i=0; i< (int)hdrarr.size(); i++) hdrarr[i].Set_Srate(srate_ksps);
  }

  // Set DAS information for latter use of Srate
  Set_Sparam(vdif_hdr.Nch(), vdif_hdr.Qbit(), vdif_hdr.Srate() );
  
  return frm_count;
  
}

const long VDIF_Frame::Read_sec( vector <uint8_t>& rawbuf, const int sec){
  vector <VDIF_Hdr> hdrarr;
  if( Tested() < 1 ||  Read_every_sec_hdr(hdrarr, sec) <=0){
      cerr << "  E Failed to VIDF_Frame::Read_sec()" << endl;
      return -1;
  };

  long pktsize = hdrarr[0].Framesize();
  size_t bufsize = nframe_sec* pktsize * sec;
  if(rawbuf.size() != bufsize)  rawbuf.resize(bufsize);
  long cnt = fread( &rawbuf[0], bufsize, 1, fp);

  long nframe = cnt*bufsize/pktsize;
  return nframe;
}

const long VDIF_Frame::Read_frame( vector <VDIF_Hdr>& hdrarr, 
				   vector <uint8_t>& datbuf,
				   const int nframe){

  if(workbuf.size() < 64) workbuf.resize(64);
  // Set Pointers 
  VDIF_Hdr *phdr =  (VDIF_Hdr*)&workbuf[0];
  size_t hdsize = phdr->Headsize();

  // Test reading frist buffer
  if(  workbuf.size() < hdsize){
    workbuf.resize(hdsize);
    phdr =  (VDIF_Hdr*)&workbuf[0];
  }
  long cnt = fread( &workbuf[0], hdsize, 1, fp);

//   fprintf(stderr,"Type1:"); phdr->Print_hd(stderr,2);
//   fprintf(stderr,"Type1b:"); phdr2->Print_hd(stderr,2);
//   for(int i=0; i<8; i++) pl[i] = ntohl(pl[i]);
//   fprintf(stderr,"Type2:"); phdr->Print_hd(stderr,2);
//   for(int i=0; i<8; i++) pl[i] = htonl(pl[i]);
//   fprintf(stderr,"Type3:"); phdr->Print_hd(stderr,2);

//   fprintf(stderr,"D: hdr1= %x %x %x %x  %x %x %x %x\n", 
// 	  workbuf[0], workbuf[1],workbuf[2],workbuf[3],
// 	  workbuf[4], workbuf[5],workbuf[6],workbuf[7] );
//   fprintf(stderr,"D: hdr2= %x %x %x %x  %x %x %x %x\n", 
// 	  workbuf[8], workbuf[9],workbuf[10],workbuf[11],
// 	  workbuf[12], workbuf[13],workbuf[14],workbuf[15] );
  if(cnt < 1){
    perror("E VDIF_Frame::Read_frame():");
    return -1;
  }
  fseek(fp,-hdsize,SEEK_CUR); // Rewind back

  long framesize = phdr->Framesize(); 
  size_t bufsize = framesize * nframe;
  long datsize = framesize - hdsize;
  if(  workbuf.size() < bufsize){
    workbuf.resize(bufsize); 
    phdr =  (VDIF_Hdr*)&workbuf[0];
  }
  
  size_t rframe = fread( &workbuf[0], framesize,nframe,fp);
  if(cnt < 1){
    perror("E VDIF_Frame::Read_frame():");
    return -1;
  }
  
  size_t datbufsize = datsize * rframe;  
  if(  datbuf.size() != datbufsize)  datbuf.resize(datbufsize);
  if(  hdrarr.size() != rframe)  hdrarr.resize(rframe);
  
  vector <uint8_t>::iterator ptr = workbuf.begin() + hdsize;
  vector <uint8_t>::iterator ptrd = datbuf.begin();
  vdif_hdr = *phdr; // Set first data to the common hdr-data in the frame

  for(int i=0; i< (int)rframe; i++){
    hdrarr[i] = *phdr;
    memcpy(&(*ptrd), &(*ptr), datsize);
    ptrd += datsize; ptr += datsize;  
    phdr = (VDIF_Hdr*)&(*ptr); ptr += hdsize;
  }
  double dt = (double)datsize/4/DAS_Info().Speed();
  Time_dat t1 = vdif_hdr.Epoch() + dt* vdif_hdr.iFrame(); 
  Set_Epoch(t1);
  
  return rframe;
}
  
  


