
#ifndef __BITUTL_HPP__
#define __BITUTL_HPP__

/** Bit Opearation Utilities .
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: bitutl.hpp 56 2013-07-03 11:40:14Z vlbi $ .
 */

#include <vector>
#include <string>

/** bit counter */
inline const int bitcnt(const uint32_t a){
  
  uint32_t x=a;
  int cnt=0;
  for(int i=0; i<32; i++){
    if(x & 0x01) cnt++;
    x >>= 1;
  }
  return cnt;
}

class Bit_hist{
  int qbit;  // 2bit default
  vector <unsigned long *> cnt_arr;
  unsigned long count; ///< Total Counter 
  int nrank; // Number of ranks;
public:
  Bit_hist(const int bit=2):qbit(2){ Init(bit);}
  void Init(const int bit=2){
    qbit = bit;
    int narr = 32 / qbit;
    if(cnt_arr.size() > 0 && cnt_arr[0] !=0){ 
      for(int i=0; i< (int)cnt_arr.size(); i++) delete [] cnt_arr[i];
    }
    cnt_arr.resize(narr);
    nrank=1;
    for(int i=0; i<qbit; i++) nrank *= 2;
    for(int i=0; i<narr;i++){
      cnt_arr[i] = new unsigned long [nrank];
      bzero((void*)cnt_arr[i],sizeof(unsigned long)*nrank);      
    }
    count=0;
  }
  void  Count(const uint32_t a);
  std::string StrHist(const int ich);
};


inline std::string Bit_hist::StrHist(const int ich){
  if(ich < -1 || ich >= (int)cnt_arr.size()){
    fprintf(stderr,"Error: ich(%d) is out of range(%lu)\n", ich, cnt_arr.size());
    return "NG";
  }

  string ret;
  char buf[512];

  if(ich == -1){
    for(int j=0; j<nrank; j++){
      sprintf(buf," [%02x] ", j);
      ret += buf;
    }
  }else{
    double ratio[nrank];
    double dcount = (double)count;
    for(int j=0; j<nrank; j++){
      ratio[j] = (double)cnt_arr[ich][j]/dcount * 100.;
      sprintf(buf,"%5.1f ", ratio[j]);
      ret += buf;
    }
  }
  return ret;  

}


inline void Bit_hist::Count(const uint32_t a){
  
  uint32_t mask = 0x03;
  switch(qbit){
  case 1: mask = 0x01; break;
  case 4: mask = 0x0f; break;
  case 8: mask = 0xff; break;
  case 2: 
  default:
    mask = 0x03; break;
  }

  uint16_t val=0;
  for(int i=0; i < (int)cnt_arr.size(); i++){
    val = (a >> (i * qbit)) & mask;
    cnt_arr[i][val] ++;
  }
  count ++;
}

#endif
