
#ifndef __HIST_G_HPP__
#define __HIST_G_HPP__
/** Histgram class
 * @Version $Rev: 56 $
 * @Author M.Sekido
 * @Date  $Date: 2013-07-03 20:40:14 +0900 (Wed, 03 Jul 2013) $
 * $Id: hist_g.hpp 56 2013-07-03 11:40:14Z vlbi $ 
 */

#include <iostream>
#include <cmath>
#include <vector>
#include <algorithm>
// using namespace std;
 
extern "C" {
#include "stdint.h"
}

template <typename T> 
class Hist_g {
  std::vector <uint32_t>  hist_arr;
  std::vector <float> ratio_arr;
  std::vector <float> level_arr[2]; ///< Upper and Lower bound
  T  step;
  T max;
  T min;
  int ndiv;
  uint32_t count;
public:
  //  Hist_g(const T tmin, const T tmax, const T stp);
  Hist_g(const T tmin, const int n, const T stp);
  Hist_g(std::vector <T>& arr):step(0){ Set_Level(arr);}
  Hist_g():min(0),max(0),step(0){;}
  /** Initializing data. 
   * This is necessary to reset hist accumulation.
   */
  void Init(){fill(hist_arr.begin(), hist_arr.end(),0);count=0;}
  /**
   * \param dat  vector array for histgram computation.
   */
  void Accm(std::vector <T>& dat);
  /**
   * \param dat Pointer to <T> array for histgram computation.
   * \param ndat Number of data for histgram computation.
   */
  void Accm(T const  *dat, const long ndat);
  std::vector <uint32_t>& HistArr() { return hist_arr;}
  std::vector <float>& RatioArr() { return ratio_arr;}
  float Ulevel(const int i) const;
  float Llevel(const int i) const;
  int Nstep() const{return hist_arr.size();}
  const int Set_Level( std::vector <T>& arr);
  string StrRatioArr();
}; 

template <typename T> 
inline float Hist_g<T>::Ulevel(const int i) const{
  if( i >= 0 && i< level_arr[1].size()) 
    return level_arr[1][i];
  else{
    std::cerr << "Error at Hist_g<T>::Ulevel(): index(" << i 
	 << ") is out of range[0," << level_arr[1].size() << "]." << endl;
    return 0;
  }
}

template <typename T> 
inline  float Hist_g<T>::Llevel(const int i) const{
  if( i>=0 && i< level_arr[0].size()) 
    return level_arr[0][i];
  else{
    std::cerr << "Error at Hist_g<T>::Ulevel(): index(" << i 
	 << ") is out of range[0," << level_arr[0].size() << "]." << endl;
    return  0;
  }
}

template <typename T> 
inline Hist_g<T>::Hist_g(const T lbound, const int n, const T stp){
  //  double range = (double)tmax - (double)tmin;
  //  int n = (int)floor(range/stp) +2;
  ndiv = n;
  min = lbound;  step = stp;
  max = lbound + stp * ndiv;  
  hist_arr.resize(n);
  level_arr[0].resize(n);  level_arr[1].resize(n);
  for(int i=0; i<n; i++){
    level_arr[0][i] = (float)(min + step*(i-1));
    level_arr[1][i] = (float)(min + step*i);
  }
  level_arr[0][0] =-INFINITY;  level_arr[1][n-1] =INFINITY;
}

template <typename T> 
inline const int Hist_g<T>::Set_Level( std::vector <T>& arr){
  int n = arr.size() +1;
  hist_arr.resize(n);
  min = arr[0]; max = arr[n-2];
  level_arr[0].resize(n);  level_arr[1].resize(n);
  for(int i=0; i<n-1; i++){
    level_arr[0][i+1] = arr[i];
    level_arr[1][i]   = arr[i];
  }
  level_arr[0][0] =-INFINITY;  level_arr[1][n-1] =INFINITY;
  return n;
}

template <typename T> 
inline void Hist_g<T>::Accm(std::vector <T>& dat){

  //  std::vector <uint32_t>::iterator ih;
  //  T val;
  for(int idat = 0; idat < (int)dat.size(); idat++){
    //    ih = hist_arr.begin();
    //    val = min;
    count++;
    for(int i=0; i< (int) hist_arr.size(); i++){
      if( dat[idat] < level_arr[1][i]){
	hist_arr[i]++; i = hist_arr.size(); 
      }
//       else if(i == (int)hist_arr.size() -2){
// 	hist_arr[i+1]++; i=hist_arr.size(); 
//       }
    }
  }
  
  if(ratio_arr.size() != hist_arr.size())  ratio_arr.resize(hist_arr.size());
  for(int i=0; i< (int)hist_arr.size(); i++){
    ratio_arr[i] = (float)hist_arr[i]/(float)count;
  }

}

template <typename T> 
inline void Hist_g<T>::Accm( T const *dat, const long ndat){

  //  std::vector <uint32_t>::iterator ih;
  //  T val;
  for(int idat = 0; idat < ndat; idat++){
    //    ih = hist_arr.begin();
    //    val = min;
    //    for(int i=0; i<hist_arr.size(); i++, val+=step){
      //      if( dat[idat] < val){
    count++;
    for(int i=0; i<hist_arr.size(); i++){
      if( dat[idat] < level_arr[1][i]){
	hist_arr[i]++; i=hist_arr.size(); 
      }
//       else if(i == hist_arr.size() -2){
// 	hist_arr[i+1]++; i=hist_arr.size(); 
//       }
    }
  }

  if(ratio_arr.size() != hist_arr.size())  ratio_arr.resize(hist_arr.size());
  for(int i=0; i< (int)hist_arr.size(); i++){
    ratio_arr[i] = (float)hist_arr[i]/(float)count;
  }

}

template <typename T> 
inline string Hist_g<T>::StrRatioArr(){
  string str;
  char buf[10];
  for(int i=0; i< (int)ratio_arr.size(); i++){
    if(i==0) sprintf(buf,"%4.1f", ratio_arr[i]*100);
    else     sprintf(buf,":%4.1f", ratio_arr[i]*100);
    str += buf;
  }
  //  str += " [%]";
  return str;
}

#endif
